// traffic generator for ddr3 control core

`define	idle		3'b000
`define	write		3'b001
`define	read		3'b010
`define	overwrite	3'b011
`define	stop		3'b100

module	ddr3_traffic_gen	(

	reset_n,
	afi_clk,
	avl_ready,
	avl_addr,
	avl_burstbegin,
	avl_rdata_valid,
	avl_write_req,
	avl_read_req,
	avl_wdata,
	avl_rdata,
	avl_size,
	init_done,
	avl_byteenable
	
							);

	output	[33:0]	avl_addr;
	output			avl_burstbegin;
	output			avl_write_req;
	output			avl_read_req;
	output	[255:0]	avl_wdata;
	output	[5:0]	avl_size;
	output	[31:0]	avl_byteenable;
	
	input			reset_n;
	input			afi_clk;
	input			avl_ready;
	input	[255:0]	avl_rdata;
	input			avl_rdata_valid;
	input			init_done;
	
	reg		[33:0]	avl_addr_buf;
	reg				avl_burstbegin_buf;
	reg				avl_write_req_buf;
	reg				avl_read_req_buf;
	reg		[255:0]	avl_wdata_buf;
	reg		[255:0]	rx_data_buf;
	reg		[2:0]	state;
	reg		[255:0]	write_data[7:0];
	reg		[33:0]	address[7:0];
	reg		[3:0]	waddr_count;
	reg		[3:0]	raddr_count;
	reg		[3:0]	read_count;
	reg		[6:0]	write_count;
	reg		[3:0]	rx_count;
	reg		[3:0]	read_data;
	reg		[5:0]	avl_size_buf;
	reg		[5:0]	burst_count;
	reg		[5:0]	burst_length[7:0];
	reg		[3:0]	burst_index;
	reg		[31:0]	avl_byteenable_buf;
	reg		[31:0]	byte_enable[7:0];
	
	assign	avl_addr		=	avl_addr_buf;
	assign	avl_burstbegin	=	avl_burstbegin_buf;
	assign	avl_write_req	=	avl_write_req_buf;
	assign	avl_read_req	=	avl_read_req_buf;
	assign	avl_wdata		=	avl_wdata_buf;
	assign	avl_size		=	avl_size_buf;
	assign	avl_byteenable	=	avl_byteenable_buf;
	
	always@	(posedge(afi_clk)	or	negedge(reset_n))
	begin
	
		if	(!reset_n)
		begin
		
			waddr_count	<=	4'b0001;
			raddr_count	<=	4'b0001;
			read_count	<=	4'b0001;
			write_count	<=	7'h01;
			burst_index	<=	4'b0001;
			burst_count	<=	6'h00;
			rx_count	<=	4'b0000;
			avl_byteenable_buf	<=	32'h00000000;
			avl_addr_buf			<=	34'h000000000;
			avl_burstbegin_buf		<=	1'b0;
			avl_write_req_buf		<=	1'b0;
			avl_read_req_buf		<=	1'b0;
			avl_wdata_buf			<=	256'h0000000000000000000000000000000000000000000000000000000000000000;
			avl_size_buf			<=	6'h00;
			$readmemb	("init_burstlength.dat",burst_length);
			$readmemh	("init_data.dat",write_data);
			$readmemb	("init_addr.dat",address);
			$readmemh	("init_byteenable.dat",byte_enable);
			read_data	=	$fopen	("rx_data.dat");
			state				<=	`idle;
		
		end
		
		case	(state)
		
			`idle	:	begin
							
							if	(init_done)
							begin
							
								avl_addr_buf		<=	address[0];
								avl_wdata_buf		<=	write_data[0];
								avl_size_buf		<=	burst_length[0];
								burst_count			<=	burst_length[0];
								state				<=	`write;
								avl_burstbegin_buf	<=	1'b1;
								avl_write_req_buf	<=	1'b1;
								avl_byteenable_buf	<=	32'hffffffff;
								
							end
							
						end
						
			`write	:	begin
			
							avl_write_req_buf	<=	1'b1;
							if(burst_count	==	avl_size)
							
								avl_burstbegin_buf	<=	1'b0;
								
							if	(avl_ready)
							begin
								
								avl_wdata_buf	<=	write_data[write_count];
								write_count		<=	write_count+1'b1;
								if(burst_count	==	6'b000001)
								begin
									
									avl_addr_buf			<=	address[waddr_count];
									avl_burstbegin_buf		<=	1'b1;
									burst_index				<=	burst_index+1'b1;
									burst_count				<=	burst_length[burst_index];
									avl_size_buf			<=	burst_length[burst_index];
									waddr_count				<=	waddr_count+1'b1;
									
								end
							
							end
							
							if	(burst_index	==	4'b1000	&&	avl_ready	&&	burst_count	==	6'b000001)
							begin
							
								state				<=	`overwrite;
								avl_write_req_buf	<=	1'b1;
								avl_read_req_buf	<=	1'b0;
								avl_addr_buf		<=	address[0];
								avl_wdata_buf		<=	256'h0000000000000000000000000000000000000000000000000000000000000000;
								avl_size_buf		<=	burst_length[0];
								burst_count			<=	burst_length[0];
								avl_byteenable_buf	<=	32'hf0000000;
								burst_index			<=	4'b0001;
								waddr_count			<=	4'b0001;
								write_count			<=	7'h01;
								avl_burstbegin_buf	<=	1'b1;
								
							end
							
						end
						
			`overwrite	:	begin
			
							avl_write_req_buf	<=	1'b1;
							if(burst_count	==	avl_size)
							
								avl_burstbegin_buf	<=	1'b0;
								
							if	(avl_ready)
							begin
								
								avl_wdata_buf			<=	256'h0000000000000000000000000000000000000000000000000000000000000000;
								// write_count		<=	write_count+1'b1;
								// burst_count		<=	burst_count-1'b1;
								if(burst_count	==	6'b000001)
								begin
									
									avl_addr_buf			<=	address[waddr_count];
									avl_byteenable_buf		<=	byte_enable[waddr_count];
									avl_burstbegin_buf		<=	1'b1;
									burst_index				<=	burst_index+1'b1;
									burst_count				<=	burst_length[burst_index];
									avl_size_buf			<=	burst_length[burst_index];
									waddr_count				<=	waddr_count+1'b1;
									
								end
							
							end
							
							if	(burst_index	==	4'b0101	&&	avl_ready	&&	burst_count	==	6'b000001)
							begin
							
								state				<=	`read;
								avl_write_req_buf	<=	1'b0;
								avl_read_req_buf	<=	1'b1;
								avl_addr_buf		<=	address[0];
								avl_wdata_buf		<=	write_data[0];
								burst_index			<=	4'b0001;
								avl_size_buf		<=	burst_length[0];
								avl_burstbegin_buf	<=	1'b1;
								avl_byteenable_buf	<=	32'hffffffff;
								
							end
							
						end
						
			`read	:	begin
			
							avl_read_req_buf		<=	1'b1;
								
							if	(avl_burstbegin_buf	==	1'b1)
							
								avl_burstbegin_buf	<=	1'b0;
							
							if	(avl_ready	&&	burst_index	<	4'b1000)
							begin
							
								avl_addr_buf			<=	address[raddr_count];
								avl_size_buf			<=	burst_length[burst_index];
								burst_index				<=	burst_index+1'b1;
								raddr_count				<=	raddr_count+1'b1;
								avl_burstbegin_buf		<=	1'b1;
								
							end
							
							if	(burst_index	==	4'b1000	&&	avl_ready)
							begin
							
								avl_read_req_buf		<=	1'b0;
								avl_write_req_buf		<=	1'b0;
								state					<=	`stop;
								
							end
							
						end
						
			`stop	:	begin
						
							avl_read_req_buf		<=	1'b0;
							avl_write_req_buf		<=	1'b0;
						
						end
			
			default	:	begin

							avl_read_req_buf		<=	1'b0;
							avl_write_req_buf		<=	1'b0;
							
							
						end
						
		endcase
		
	end
	
	always@	(posedge(afi_clk))
	begin
	
		if	(reset_n)
		begin
			
			if	(avl_rdata_valid)
			begin
			
				$fdisplay	(read_data,"%h",avl_rdata);
				rx_count	<=	rx_count+1;
				
			end
			
		end
		
	end
		
endmodule

			
							
							
							